/*

   Copyright 2002, 2007 Kevin Ryde

   This file is part of MPM.

   MPM is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
   Foundation; either version 2, or (at your option) any later version.

   MPM is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
   FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
   details.

   You should have received a copy of the GNU General Public License along
   with MPM; see the file COPYING.  If not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301
   USA.  */

#include <stdio.h>
#include <string.h>
#include "gmp.h"

#include "mpm.h"


void
table (int (*fun) (mpr_srcptr, mpm_srcptr))
{
  mpz_t  i;
  mpm_t  m;
  mpr_t  a;

  mpz_init_set_ui (i, 3L);
  for (;;)
    {
      if (! mpz_probab_prime_p (i, 10))
        {
          mpm_init (m, i);
          mpr_init_set_ui (a, 2L, m);
          if ((*fun) (a, m))
            gmp_printf ("%Zd\n", i);
          mpr_clear (a, m);
          mpm_clear (m);
        }
      mpz_add_ui (i, i, 2L);
    }
}


int
main (int argc, char *argv[])
{
  mpz_t  N;
  mpm_t  m;
  mpr_t  a;
  int    i;

  mpz_init (N);

  for (i = 1; i < argc; i++)
    {
      if (strcmp (argv[i], "t") == 0)
        {
          table (mpr_prp_p);
          continue;
        }
      if (strcmp (argv[i], "te") == 0)
        {
          table (mpr_eprp_p);
          continue;
        }
      if (strcmp (argv[i], "ts") == 0)
        {
          table (mpr_sprp_p);
          continue;
        }

      if (mpz_set_str (N, argv[i], 0) != 0)
        {
          printf ("Invalid argument: %s\n", argv[i]);
          continue;
        }

      gmp_printf ("%Zd:\n", N);
      if (mpz_probab_prime_p (N, 10))
        {
          printf ("  prime\n");
          continue;
        }

      mpm_init (m, N);
      mpr_init (a, m);

#define TRY(name, function, base)                       \
      do {                                              \
        printf ("  %s base %d: ", name, base);          \
        mpr_set_ui (a, (unsigned long) base, m);        \
        puts (function (a, m) ? "yes" : "no");          \
      } while (0)

      TRY ("pseudoprime       ", mpr_prp_p,  2);
      TRY ("euler pseudoprime ", mpr_eprp_p, 2);
      TRY ("strong pseudoprime", mpr_sprp_p, 2);
      TRY ("pseudoprime       ", mpr_prp_p,  3);
      TRY ("euler pseudoprime ", mpr_eprp_p, 3);
      TRY ("strong pseudoprime", mpr_sprp_p, 3);

      mpr_clear (a, m);
      mpm_clear (m);
    }

  return 0;
}
